@@ -1,3 +1,5 @@
+from formskit.messages import Message
+
 from haplugin.jinja2 import Jinja2HelperMany
 
 
@@ -10,13 +12,13 @@ class FormWidget(Jinja2HelperMany):
         self.form = form
 
     def get_id(self, name):
-        return '%s_%s' % (self.form.name, name)
+        return '%s_%s' % (self.form.get_name(), name)
 
     def begin(self, id_=None, style=None):
         data = {}
         data['action'] = getattr(self.form, 'action', None)
         data['id'] = id_
-        data['name'] = self.form.name
+        data['name'] = self.form.get_name()
         data['style'] = style
         return self.render_for('begin.jinja2', data)
 
@@ -32,23 +34,31 @@ class FormWidget(Jinja2HelperMany):
     def select(self, name, disabled=False, autofocus=False):
         return self._input('select', name, disabled, autofocus)
 
-    def _input(self, input_type, name, disabled=False, autofocus=False):
+    def _base_input(self, name):
         data = {}
-        data['name'] = name
+        data['name'] = self.form.fields[name].get_name().decode('utf8')
+        data['value'] = self.form.get_value(name, default='')
+        data['field'] = self.form.fields[name]
+        return data
+
+    def _input(self, input_type, name, disabled=False, autofocus=False):
+        data = self._base_input(name)
+        field = data['field']
+
         data['id'] = self.get_id(name)
-        data['label'] = self.form.get_label(name)
-        data['error'] = self.form.get_error(name)
-        data['message'] = self.form.get_message(name)
-        data['value'] = self.form.get_value(name) or ''
+        data['label'] = field.label
+        data['error'] = field.error
+        data['messages'] = [
+            self._translate(message) for message in field.messages
+        ]
+        data['value_message'] = self._translate(
+            field.get_value_error(default=None))
         data['disabled'] = disabled
         data['autofocus'] = autofocus
-        data['field'] = self.form.field_patterns[name]
         return self.render_for(input_type + '.jinja2', data)
 
     def hidden(self, name):
-        data = {}
-        data['name'] = name
-        data['value'] = self.form.get_value(name) or ''
+        data = self._base_input(name)
         return self.render_for('hidden.jinja2', data)
 
     def csrf_token(self):
@@ -63,6 +73,13 @@ class FormWidget(Jinja2HelperMany):
 
     def error(self):
         data = {}
-        data['error'] = self.form.error
-        data['message'] = self.form.message
+        data['error'] = True if self.form.success is False else False
+        data['message'] = self._translate(self.form.message)
         return self.render_for('error.jinja2', data)
+
+    def _translate(self, message):
+        if message:
+            cls = self.settings.get('form_message', Message)
+            return cls(message)
+        else:
+            return None
