@@ -1,7 +1,7 @@
 from pyramid.session import check_csrf_token
 from hatak.unpackrequest import unpack
 
-from formskit import Form, Field
+from formskit import Form
 from formskit.formvalidators import FormValidator
 
 
@@ -12,15 +12,19 @@ class PostForm(Form):
         unpack(self, self.request)
         super().__init__()
 
-        self.addField(Field('csrf_token'))
-        self.addFormValidator(CsrfMustMatch())
+        self.add_form_validator(CsrfMustMatch())
+        self.init_csrf()
 
-    def __call__(self, initial_data=None):
-        initial_data = initial_data or {}
-        initial_data['csrf_token'] = [self.request.session.get_csrf_token()]
-        return super().__call__(
-            self.request.POST.dict_of_lists(),
-            initial_data=initial_data)
+    def reset(self):
+        super().reset()
+        self.init_csrf()
+
+    def init_csrf(self):
+        self.add_field('csrf_token')
+        self.set_value('csrf_token', self.session.get_csrf_token())
+
+    def __call__(self):
+        return super().__call__(self.request.POST.dict_of_lists())
 
 
 class CsrfMustMatch(FormValidator):
@@ -28,4 +32,5 @@ class CsrfMustMatch(FormValidator):
     message = "CSRF token do not match!"
 
     def validate(self):
+        self.form.POST['csrf_token'] = self.form.get_value('csrf_token')
         return check_csrf_token(self.form.request, raises=False)
