@@ -1,6 +1,7 @@
 from hatak.plugin import Plugin
 from hatak.controller import ControllerPlugin
 from haplugin.jinja2 import Jinja2Plugin
+from haplugin.beaker import BeakerPlugin
 
 from .helpers import FormWidget
 
@@ -15,6 +16,7 @@ class FormPlugin(Plugin):
 
     def validate_plugin(self):
         self.app._validate_dependency_plugin(Jinja2Plugin)
+        self.app._validate_dependency_plugin(BeakerPlugin)
 
 
 class FormControllerPlugin(ControllerPlugin):
@@ -23,6 +25,7 @@ class FormControllerPlugin(ControllerPlugin):
         self.add_method('add_form')
 
     def add_form(self, formcls, name='form', *args, **kwargs):
+        widget = kwargs.pop('widget', self.parent.widget)
         form = formcls(self.request, *args, **kwargs)
-        self.controller.add_helper(name, self.parent.widget, form)
+        self.controller.add_helper(name, widget, form)
         return form
